"""
Pydantic models for mailchimp connector.

This module contains Pydantic models used for authentication configuration
and response envelope types.
"""

from __future__ import annotations

from pydantic import BaseModel, ConfigDict, Field
from typing import TypeVar, Generic, Union, Any
from typing import Optional

# Authentication configuration

class MailchimpAuthConfig(BaseModel):
    """API Key Authentication"""

    model_config = ConfigDict(extra="forbid")

    api_key: str
    """Your Mailchimp API key. You can find this in your Mailchimp account under Account > Extras > API keys."""
    data_center: Optional[str] = None
    """The data center for your Mailchimp account. This is the suffix of your API key (e.g., 'us6' if your API key ends with '-us6')."""

# ===== RESPONSE TYPE DEFINITIONS (PYDANTIC) =====

class CampaignTracking(BaseModel):
    """The tracking options for a campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    opens: Union[bool | None, Any] = Field(default=None, description="Whether to track opens")
    """Whether to track opens"""
    html_clicks: Union[bool | None, Any] = Field(default=None, description="Whether to track clicks in the HTML version of the campaign")
    """Whether to track clicks in the HTML version of the campaign"""
    text_clicks: Union[bool | None, Any] = Field(default=None, description="Whether to track clicks in the plain-text version of the campaign")
    """Whether to track clicks in the plain-text version of the campaign"""
    goal_tracking: Union[bool | None, Any] = Field(default=None, description="Whether to enable Goal tracking")
    """Whether to enable Goal tracking"""
    ecomm360: Union[bool | None, Any] = Field(default=None, description="Whether to enable eCommerce360 tracking")
    """Whether to enable eCommerce360 tracking"""
    google_analytics: Union[str | None, Any] = Field(default=None, description="The custom slug for Google Analytics tracking")
    """The custom slug for Google Analytics tracking"""
    clicktale: Union[str | None, Any] = Field(default=None, description="The custom slug for ClickTale tracking")
    """The custom slug for ClickTale tracking"""

class CampaignRecipients(BaseModel):
    """List settings for the campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    list_id: Union[str | None, Any] = Field(default=None, description="The unique list id")
    """The unique list id"""
    list_is_active: Union[bool | None, Any] = Field(default=None, description="The status of the list used")
    """The status of the list used"""
    list_name: Union[str | None, Any] = Field(default=None, description="The name of the list")
    """The name of the list"""
    segment_text: Union[str | None, Any] = Field(default=None, description="A description of the segment used for the campaign")
    """A description of the segment used for the campaign"""
    recipient_count: Union[int | None, Any] = Field(default=None, description="Count of the recipients on the associated list")
    """Count of the recipients on the associated list"""

class CampaignReportSummaryEcommerce(BaseModel):
    """E-Commerce stats for a campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_orders: Union[int | None, Any] = Field(default=None, description="The total orders for a campaign")
    """The total orders for a campaign"""
    total_spent: Union[float | None, Any] = Field(default=None, description="The total spent for a campaign")
    """The total spent for a campaign"""
    total_revenue: Union[float | None, Any] = Field(default=None, description="The total revenue for a campaign")
    """The total revenue for a campaign"""

class CampaignReportSummary(BaseModel):
    """For sent campaigns, a summary of opens, clicks, and e-commerce data"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    opens: Union[int | None, Any] = Field(default=None, description="The total number of opens for a campaign")
    """The total number of opens for a campaign"""
    unique_opens: Union[int | None, Any] = Field(default=None, description="The number of unique opens")
    """The number of unique opens"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The number of unique opens divided by the total number of successful deliveries")
    """The number of unique opens divided by the total number of successful deliveries"""
    clicks: Union[int | None, Any] = Field(default=None, description="The total number of clicks for an campaign")
    """The total number of clicks for an campaign"""
    subscriber_clicks: Union[int | None, Any] = Field(default=None, description="The number of unique clicks")
    """The number of unique clicks"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The number of unique clicks divided by the total number of successful deliveries")
    """The number of unique clicks divided by the total number of successful deliveries"""
    ecommerce: Union[CampaignReportSummaryEcommerce | None, Any] = Field(default=None, description="E-Commerce stats for a campaign")
    """E-Commerce stats for a campaign"""

class CampaignDeliveryStatus(BaseModel):
    """Updates on campaigns in the process of sending"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    enabled: Union[bool | None, Any] = Field(default=None, description="Whether Campaign Delivery Status is enabled for this account and target campaign")
    """Whether Campaign Delivery Status is enabled for this account and target campaign"""
    can_cancel: Union[bool | None, Any] = Field(default=None, description="Whether a campaign send can be canceled")
    """Whether a campaign send can be canceled"""
    status: Union[str | None, Any] = Field(default=None, description="The current state of a campaign delivery")
    """The current state of a campaign delivery"""
    emails_sent: Union[int | None, Any] = Field(default=None, description="The total number of emails confirmed sent for this campaign so far")
    """The total number of emails confirmed sent for this campaign so far"""
    emails_canceled: Union[int | None, Any] = Field(default=None, description="The total number of emails canceled for this campaign")
    """The total number of emails canceled for this campaign"""

class CampaignSettings(BaseModel):
    """The settings for your campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    subject_line: Union[str | None, Any] = Field(default=None, description="The subject line for the campaign")
    """The subject line for the campaign"""
    preview_text: Union[str | None, Any] = Field(default=None, description="The preview text for the campaign")
    """The preview text for the campaign"""
    title: Union[str | None, Any] = Field(default=None, description="The title of the campaign")
    """The title of the campaign"""
    from_name: Union[str | None, Any] = Field(default=None, description="The from name on the campaign")
    """The from name on the campaign"""
    reply_to: Union[str | None, Any] = Field(default=None, description="The reply-to email address for the campaign")
    """The reply-to email address for the campaign"""
    use_conversation: Union[bool | None, Any] = Field(default=None, description="Use Mailchimp Conversation feature to manage out-of-office replies")
    """Use Mailchimp Conversation feature to manage out-of-office replies"""
    to_name: Union[str | None, Any] = Field(default=None, description="The campaign's custom to name")
    """The campaign's custom to name"""
    folder_id: Union[str | None, Any] = Field(default=None, description="If the campaign is listed in a folder")
    """If the campaign is listed in a folder"""
    authenticate: Union[bool | None, Any] = Field(default=None, description="Whether Mailchimp authenticated the campaign")
    """Whether Mailchimp authenticated the campaign"""
    auto_footer: Union[bool | None, Any] = Field(default=None, description="Automatically append Mailchimp's default footer to the campaign")
    """Automatically append Mailchimp's default footer to the campaign"""
    inline_css: Union[bool | None, Any] = Field(default=None, description="Automatically inline the CSS included with the campaign content")
    """Automatically inline the CSS included with the campaign content"""
    auto_tweet: Union[bool | None, Any] = Field(default=None, description="Automatically tweet a link to the campaign archive page when the campaign is sent")
    """Automatically tweet a link to the campaign archive page when the campaign is sent"""
    fb_comments: Union[bool | None, Any] = Field(default=None, description="Allows Facebook comments on the campaign")
    """Allows Facebook comments on the campaign"""
    timewarp: Union[bool | None, Any] = Field(default=None, description="Send this campaign using Timewarp")
    """Send this campaign using Timewarp"""
    template_id: Union[int | None, Any] = Field(default=None, description="The id for the template used in this campaign")
    """The id for the template used in this campaign"""
    drag_and_drop: Union[bool | None, Any] = Field(default=None, description="Whether the campaign uses the drag-and-drop editor")
    """Whether the campaign uses the drag-and-drop editor"""

class Campaign(BaseModel):
    """A summary of an individual campaign's settings and content"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    web_id: Union[int | None, Any] = Field(default=None)
    parent_campaign_id: Union[str | None, Any] = Field(default=None)
    type: Union[str | None, Any] = Field(default=None)
    create_time: Union[str | None, Any] = Field(default=None)
    archive_url: Union[str | None, Any] = Field(default=None)
    long_archive_url: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    emails_sent: Union[int | None, Any] = Field(default=None)
    send_time: Union[str | None, Any] = Field(default=None)
    content_type: Union[str | None, Any] = Field(default=None)
    needs_block_refresh: Union[bool | None, Any] = Field(default=None)
    resendable: Union[bool | None, Any] = Field(default=None)
    recipients: Union[CampaignRecipients | None, Any] = Field(default=None)
    settings: Union[CampaignSettings | None, Any] = Field(default=None)
    tracking: Union[CampaignTracking | None, Any] = Field(default=None)
    report_summary: Union[CampaignReportSummary | None, Any] = Field(default=None)
    delivery_status: Union[CampaignDeliveryStatus | None, Any] = Field(default=None)

class CampaignsList(BaseModel):
    """A collection of campaigns"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    campaigns: Union[list[Campaign], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class ListContact(BaseModel):
    """Contact information displayed in campaign footers to comply with international spam laws"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    company: Union[str | None, Any] = Field(default=None, description="The company name for the list")
    """The company name for the list"""
    address1: Union[str | None, Any] = Field(default=None, description="The street address for the list contact")
    """The street address for the list contact"""
    address2: Union[str | None, Any] = Field(default=None, description="The street address for the list contact")
    """The street address for the list contact"""
    city: Union[str | None, Any] = Field(default=None, description="The city for the list contact")
    """The city for the list contact"""
    state: Union[str | None, Any] = Field(default=None, description="The state for the list contact")
    """The state for the list contact"""
    zip: Union[str | None, Any] = Field(default=None, description="The postal or zip code for the list contact")
    """The postal or zip code for the list contact"""
    country: Union[str | None, Any] = Field(default=None, description="A two-character ISO3166 country code")
    """A two-character ISO3166 country code"""
    phone: Union[str | None, Any] = Field(default=None, description="The phone number for the list contact")
    """The phone number for the list contact"""

class ListCampaignDefaults(BaseModel):
    """Default values for campaigns created for this list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    from_name: Union[str | None, Any] = Field(default=None, description="The default from name for campaigns sent to this list")
    """The default from name for campaigns sent to this list"""
    from_email: Union[str | None, Any] = Field(default=None, description="The default from email for campaigns sent to this list")
    """The default from email for campaigns sent to this list"""
    subject: Union[str | None, Any] = Field(default=None, description="The default subject line for campaigns sent to this list")
    """The default subject line for campaigns sent to this list"""
    language: Union[str | None, Any] = Field(default=None, description="The default language for this list's forms")
    """The default language for this list's forms"""

class ListStats(BaseModel):
    """Stats for the list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    member_count: Union[int | None, Any] = Field(default=None, description="The number of active members in the list")
    """The number of active members in the list"""
    total_contacts: Union[int | None, Any] = Field(default=None, description="The number of contacts in the list, including subscribed, unsubscribed, pending, cleaned, deleted, transactional, and those that need to be reconfirmed")
    """The number of contacts in the list, including subscribed, unsubscribed, pending, cleaned, deleted, transactional, and those that need to be reconfirmed"""
    unsubscribe_count: Union[int | None, Any] = Field(default=None, description="The number of members who have unsubscribed from the list")
    """The number of members who have unsubscribed from the list"""
    cleaned_count: Union[int | None, Any] = Field(default=None, description="The number of members cleaned from the list")
    """The number of members cleaned from the list"""
    member_count_since_send: Union[int | None, Any] = Field(default=None, description="The number of active members in the list since the last campaign was sent")
    """The number of active members in the list since the last campaign was sent"""
    unsubscribe_count_since_send: Union[int | None, Any] = Field(default=None, description="The number of members who have unsubscribed since the last campaign was sent")
    """The number of members who have unsubscribed since the last campaign was sent"""
    cleaned_count_since_send: Union[int | None, Any] = Field(default=None, description="The number of members cleaned from the list since the last campaign was sent")
    """The number of members cleaned from the list since the last campaign was sent"""
    campaign_count: Union[int | None, Any] = Field(default=None, description="The number of campaigns in any status that use this list")
    """The number of campaigns in any status that use this list"""
    campaign_last_sent: Union[str | None, Any] = Field(default=None, description="The date and time the last campaign was sent to this list")
    """The date and time the last campaign was sent to this list"""
    merge_field_count: Union[int | None, Any] = Field(default=None, description="The number of merge fields for this list")
    """The number of merge fields for this list"""
    avg_sub_rate: Union[float | None, Any] = Field(default=None, description="The average number of subscriptions per month for the list")
    """The average number of subscriptions per month for the list"""
    avg_unsub_rate: Union[float | None, Any] = Field(default=None, description="The average number of unsubscriptions per month for the list")
    """The average number of unsubscriptions per month for the list"""
    target_sub_rate: Union[float | None, Any] = Field(default=None, description="The target number of subscriptions per month for the list to keep it growing")
    """The target number of subscriptions per month for the list to keep it growing"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The average open rate for campaigns sent to this list")
    """The average open rate for campaigns sent to this list"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The average click rate for campaigns sent to this list")
    """The average click rate for campaigns sent to this list"""
    last_sub_date: Union[str | None, Any] = Field(default=None, description="The date and time of the last time someone subscribed to this list")
    """The date and time of the last time someone subscribed to this list"""
    last_unsub_date: Union[str | None, Any] = Field(default=None, description="The date and time of the last time someone unsubscribed from this list")
    """The date and time of the last time someone unsubscribed from this list"""

class List(BaseModel):
    """Information about a specific list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    web_id: Union[int | None, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)
    contact: Union[ListContact | None, Any] = Field(default=None)
    permission_reminder: Union[str | None, Any] = Field(default=None)
    use_archive_bar: Union[bool | None, Any] = Field(default=None)
    campaign_defaults: Union[ListCampaignDefaults | None, Any] = Field(default=None)
    notify_on_subscribe: Union[str | None, Any] = Field(default=None)
    notify_on_unsubscribe: Union[str | None, Any] = Field(default=None)
    date_created: Union[str | None, Any] = Field(default=None)
    list_rating: Union[int | None, Any] = Field(default=None)
    email_type_option: Union[bool | None, Any] = Field(default=None)
    subscribe_url_short: Union[str | None, Any] = Field(default=None)
    subscribe_url_long: Union[str | None, Any] = Field(default=None)
    beamer_address: Union[str | None, Any] = Field(default=None)
    visibility: Union[str | None, Any] = Field(default=None)
    double_optin: Union[bool | None, Any] = Field(default=None)
    has_welcome: Union[bool | None, Any] = Field(default=None)
    marketing_permissions: Union[bool | None, Any] = Field(default=None)
    stats: Union[ListStats | None, Any] = Field(default=None)

class ListsList(BaseModel):
    """A collection of subscriber lists for this account"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    lists: Union[list[List], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class ListMemberTagsItem(BaseModel):
    """Nested schema for ListMember.tags_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[int | None, Any] = Field(default=None, description="The tag id")
    """The tag id"""
    name: Union[str | None, Any] = Field(default=None, description="The name of the tag")
    """The name of the tag"""

class ListMemberStatsEcommerceData(BaseModel):
    """Ecommerce stats for the list member if the list is attached to a store"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_revenue: Union[float | None, Any] = Field(default=None, description="The total revenue the list member has brought in")
    """The total revenue the list member has brought in"""
    number_of_orders: Union[float | None, Any] = Field(default=None, description="The total number of orders placed by the list member")
    """The total number of orders placed by the list member"""
    currency_code: Union[str | None, Any] = Field(default=None, description="The three-letter ISO 4217 code for the currency that the store accepts")
    """The three-letter ISO 4217 code for the currency that the store accepts"""

class ListMemberStats(BaseModel):
    """Open and click rates for this subscriber"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    avg_open_rate: Union[float | None, Any] = Field(default=None, description="A subscriber's average open rate")
    """A subscriber's average open rate"""
    avg_click_rate: Union[float | None, Any] = Field(default=None, description="A subscriber's average clickthrough rate")
    """A subscriber's average clickthrough rate"""
    ecommerce_data: Union[ListMemberStatsEcommerceData | None, Any] = Field(default=None, description="Ecommerce stats for the list member if the list is attached to a store")
    """Ecommerce stats for the list member if the list is attached to a store"""

class ListMemberLocation(BaseModel):
    """Subscriber location information"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    latitude: Union[float | None, Any] = Field(default=None, description="The location latitude")
    """The location latitude"""
    longitude: Union[float | None, Any] = Field(default=None, description="The location longitude")
    """The location longitude"""
    gmtoff: Union[int | None, Any] = Field(default=None, description="The time difference in hours from GMT")
    """The time difference in hours from GMT"""
    dstoff: Union[int | None, Any] = Field(default=None, description="The offset for timezones where daylight saving time is observed")
    """The offset for timezones where daylight saving time is observed"""
    country_code: Union[str | None, Any] = Field(default=None, description="The unique code for the location country")
    """The unique code for the location country"""
    timezone: Union[str | None, Any] = Field(default=None, description="The timezone for the location")
    """The timezone for the location"""
    region: Union[str | None, Any] = Field(default=None, description="The region for the location")
    """The region for the location"""

class ListMember(BaseModel):
    """Individuals who are currently or have been previously subscribed to this list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    email_address: Union[str | None, Any] = Field(default=None)
    unique_email_id: Union[str | None, Any] = Field(default=None)
    contact_id: Union[str | None, Any] = Field(default=None)
    full_name: Union[str | None, Any] = Field(default=None)
    web_id: Union[int | None, Any] = Field(default=None)
    email_type: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    unsubscribe_reason: Union[str | None, Any] = Field(default=None)
    consents_to_one_to_one_messaging: Union[bool | None, Any] = Field(default=None)
    merge_fields: Union[dict[str, Any] | None, Any] = Field(default=None)
    interests: Union[dict[str, bool] | None, Any] = Field(default=None)
    stats: Union[ListMemberStats | None, Any] = Field(default=None)
    ip_signup: Union[str | None, Any] = Field(default=None)
    timestamp_signup: Union[str | None, Any] = Field(default=None)
    ip_opt: Union[str | None, Any] = Field(default=None)
    timestamp_opt: Union[str | None, Any] = Field(default=None)
    member_rating: Union[int | None, Any] = Field(default=None)
    last_changed: Union[str | None, Any] = Field(default=None)
    language: Union[str | None, Any] = Field(default=None)
    vip: Union[bool | None, Any] = Field(default=None)
    email_client: Union[str | None, Any] = Field(default=None)
    location: Union[ListMemberLocation | None, Any] = Field(default=None)
    source: Union[str | None, Any] = Field(default=None)
    tags_count: Union[int | None, Any] = Field(default=None)
    tags: Union[list[ListMemberTagsItem] | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)

class ListMembersList(BaseModel):
    """Manage members of a specific Mailchimp list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    members: Union[list[ListMember], Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class ReportBounces(BaseModel):
    """An object describing the bounce summary for the campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    hard_bounces: Union[int | None, Any] = Field(default=None, description="The total number of hard bounced email addresses")
    """The total number of hard bounced email addresses"""
    soft_bounces: Union[int | None, Any] = Field(default=None, description="The total number of soft bounced email addresses")
    """The total number of soft bounced email addresses"""
    syntax_errors: Union[int | None, Any] = Field(default=None, description="The total number of addresses that were syntax-related bounces")
    """The total number of addresses that were syntax-related bounces"""

class ReportClicks(BaseModel):
    """An object describing the click activity for the campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    clicks_total: Union[int | None, Any] = Field(default=None, description="The total number of clicks for the campaign")
    """The total number of clicks for the campaign"""
    unique_clicks: Union[int | None, Any] = Field(default=None, description="The total number of unique clicks for links across a campaign")
    """The total number of unique clicks for links across a campaign"""
    unique_subscriber_clicks: Union[int | None, Any] = Field(default=None, description="The total number of subscribers who clicked on a campaign")
    """The total number of subscribers who clicked on a campaign"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The number of unique clicks divided by the total number of successful deliveries")
    """The number of unique clicks divided by the total number of successful deliveries"""
    last_click: Union[str | None, Any] = Field(default=None, description="The date and time of the last recorded click for the campaign")
    """The date and time of the last recorded click for the campaign"""

class ReportOpens(BaseModel):
    """An object describing the open activity for the campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    opens_total: Union[int | None, Any] = Field(default=None, description="The total number of opens for a campaign")
    """The total number of opens for a campaign"""
    unique_opens: Union[int | None, Any] = Field(default=None, description="The total number of unique opens")
    """The total number of unique opens"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The number of unique opens divided by the total number of successful deliveries")
    """The number of unique opens divided by the total number of successful deliveries"""
    last_open: Union[str | None, Any] = Field(default=None, description="The date and time of the last recorded open")
    """The date and time of the last recorded open"""

class ReportIndustryStats(BaseModel):
    """The average campaign statistics for your industry"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    type: Union[str | None, Any] = Field(default=None, description="The type of business industry associated with your account")
    """The type of business industry associated with your account"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The industry open rate")
    """The industry open rate"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The industry click rate")
    """The industry click rate"""
    bounce_rate: Union[float | None, Any] = Field(default=None, description="The industry bounce rate")
    """The industry bounce rate"""
    unopen_rate: Union[float | None, Any] = Field(default=None, description="The industry unopened rate")
    """The industry unopened rate"""
    unsub_rate: Union[float | None, Any] = Field(default=None, description="The industry unsubscribe rate")
    """The industry unsubscribe rate"""
    abuse_rate: Union[float | None, Any] = Field(default=None, description="The industry abuse rate")
    """The industry abuse rate"""

class ReportEcommerce(BaseModel):
    """E-Commerce stats for a campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_orders: Union[int | None, Any] = Field(default=None, description="The total orders for a campaign")
    """The total orders for a campaign"""
    total_spent: Union[float | None, Any] = Field(default=None, description="The total spent for a campaign")
    """The total spent for a campaign"""
    total_revenue: Union[float | None, Any] = Field(default=None, description="The total revenue for a campaign")
    """The total revenue for a campaign"""

class ReportFacebookLikes(BaseModel):
    """An object describing campaign engagement on Facebook"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    recipient_likes: Union[int | None, Any] = Field(default=None, description="The number of recipients who liked the campaign on Facebook")
    """The number of recipients who liked the campaign on Facebook"""
    unique_likes: Union[int | None, Any] = Field(default=None, description="The number of unique likes")
    """The number of unique likes"""
    facebook_likes: Union[int | None, Any] = Field(default=None, description="The number of Facebook likes for the campaign")
    """The number of Facebook likes for the campaign"""

class ReportForwards(BaseModel):
    """An object describing the forwards and forward activity for the campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    forwards_count: Union[int | None, Any] = Field(default=None, description="How many times the campaign has been forwarded")
    """How many times the campaign has been forwarded"""
    forwards_opens: Union[int | None, Any] = Field(default=None, description="How many times the forwarded campaign has been opened")
    """How many times the forwarded campaign has been opened"""

class ReportListStats(BaseModel):
    """The average campaign statistics for your list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    sub_rate: Union[float | None, Any] = Field(default=None, description="The average number of subscriptions per month for the list")
    """The average number of subscriptions per month for the list"""
    unsub_rate: Union[float | None, Any] = Field(default=None, description="The average number of unsubscriptions per month for the list")
    """The average number of unsubscriptions per month for the list"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The average open rate for campaigns sent to this list")
    """The average open rate for campaigns sent to this list"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The average click rate for campaigns sent to this list")
    """The average click rate for campaigns sent to this list"""

class ReportDeliveryStatus(BaseModel):
    """Updates on campaigns in the process of sending"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    enabled: Union[bool | None, Any] = Field(default=None, description="Whether Campaign Delivery Status is enabled for this account and target campaign")
    """Whether Campaign Delivery Status is enabled for this account and target campaign"""

class Report(BaseModel):
    """Report details about a sent campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    campaign_title: Union[str | None, Any] = Field(default=None)
    type: Union[str | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    list_is_active: Union[bool | None, Any] = Field(default=None)
    list_name: Union[str | None, Any] = Field(default=None)
    subject_line: Union[str | None, Any] = Field(default=None)
    preview_text: Union[str | None, Any] = Field(default=None)
    emails_sent: Union[int | None, Any] = Field(default=None)
    abuse_reports: Union[int | None, Any] = Field(default=None)
    unsubscribed: Union[int | None, Any] = Field(default=None)
    send_time: Union[str | None, Any] = Field(default=None)
    rss_last_send: Union[str | None, Any] = Field(default=None)
    bounces: Union[ReportBounces | None, Any] = Field(default=None)
    forwards: Union[ReportForwards | None, Any] = Field(default=None)
    opens: Union[ReportOpens | None, Any] = Field(default=None)
    clicks: Union[ReportClicks | None, Any] = Field(default=None)
    facebook_likes: Union[ReportFacebookLikes | None, Any] = Field(default=None)
    industry_stats: Union[ReportIndustryStats | None, Any] = Field(default=None)
    list_stats: Union[ReportListStats | None, Any] = Field(default=None)
    ecommerce: Union[ReportEcommerce | None, Any] = Field(default=None)
    delivery_status: Union[ReportDeliveryStatus | None, Any] = Field(default=None)

class ReportsList(BaseModel):
    """A list of reports containing campaigns marked as Sent"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    reports: Union[list[Report], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class EmailActivityActivityItem(BaseModel):
    """Nested schema for EmailActivity.activity_item"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    action: Union[str | None, Any] = Field(default=None, description="One of the following actions open, click, or bounce")
    """One of the following actions open, click, or bounce"""
    timestamp: Union[str | None, Any] = Field(default=None, description="The date and time recorded for the action")
    """The date and time recorded for the action"""
    url: Union[str | None, Any] = Field(default=None, description="If the action is a click, the URL on which the member clicked")
    """If the action is a click, the URL on which the member clicked"""
    type: Union[str | None, Any] = Field(default=None, description="If the action is a bounce, the type of bounce received hard, soft, or blocked")
    """If the action is a bounce, the type of bounce received hard, soft, or blocked"""
    ip: Union[str | None, Any] = Field(default=None, description="The IP address recorded for the action")
    """The IP address recorded for the action"""

class EmailActivity(BaseModel):
    """A summary of the email activity for a campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    campaign_id: Union[str | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    list_is_active: Union[bool | None, Any] = Field(default=None)
    email_id: Union[str | None, Any] = Field(default=None)
    email_address: Union[str | None, Any] = Field(default=None)
    activity: Union[list[EmailActivityActivityItem] | None, Any] = Field(default=None)

class EmailActivityList(BaseModel):
    """A list of member's subscriber activity in a specific campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    emails: Union[list[EmailActivity], Any] = Field(default=None)
    campaign_id: Union[str | None, Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class AutomationReportSummary(BaseModel):
    """A summary of opens and clicks for sent campaigns"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    opens: Union[int | None, Any] = Field(default=None, description="The total number of opens for a campaign")
    """The total number of opens for a campaign"""
    unique_opens: Union[int | None, Any] = Field(default=None, description="The number of unique opens")
    """The number of unique opens"""
    open_rate: Union[float | None, Any] = Field(default=None, description="The number of unique opens divided by the total number of successful deliveries")
    """The number of unique opens divided by the total number of successful deliveries"""
    clicks: Union[int | None, Any] = Field(default=None, description="The total number of clicks for an campaign")
    """The total number of clicks for an campaign"""
    subscriber_clicks: Union[int | None, Any] = Field(default=None, description="The number of unique clicks")
    """The number of unique clicks"""
    click_rate: Union[float | None, Any] = Field(default=None, description="The number of unique clicks divided by the total number of successful deliveries")
    """The number of unique clicks divided by the total number of successful deliveries"""

class AutomationTracking(BaseModel):
    """The tracking options for the Automation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    opens: Union[bool | None, Any] = Field(default=None, description="Whether to track opens")
    """Whether to track opens"""
    html_clicks: Union[bool | None, Any] = Field(default=None, description="Whether to track clicks in the HTML version")
    """Whether to track clicks in the HTML version"""
    text_clicks: Union[bool | None, Any] = Field(default=None, description="Whether to track clicks in the plain-text version")
    """Whether to track clicks in the plain-text version"""
    goal_tracking: Union[bool | None, Any] = Field(default=None, description="Whether to enable Goal tracking")
    """Whether to enable Goal tracking"""
    ecomm360: Union[bool | None, Any] = Field(default=None, description="Whether to enable eCommerce360 tracking")
    """Whether to enable eCommerce360 tracking"""
    google_analytics: Union[str | None, Any] = Field(default=None, description="The custom slug for Google Analytics tracking")
    """The custom slug for Google Analytics tracking"""
    clicktale: Union[str | None, Any] = Field(default=None, description="The custom slug for ClickTale tracking")
    """The custom slug for ClickTale tracking"""

class AutomationSettings(BaseModel):
    """The settings for the Automation workflow"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    title: Union[str | None, Any] = Field(default=None, description="The title of the Automation")
    """The title of the Automation"""
    from_name: Union[str | None, Any] = Field(default=None, description="The from name for the Automation")
    """The from name for the Automation"""
    reply_to: Union[str | None, Any] = Field(default=None, description="The reply-to email address for the Automation")
    """The reply-to email address for the Automation"""
    use_conversation: Union[bool | None, Any] = Field(default=None, description="Whether to use Mailchimp Conversation feature")
    """Whether to use Mailchimp Conversation feature"""
    to_name: Union[str | None, Any] = Field(default=None, description="The Automation's custom to name")
    """The Automation's custom to name"""
    authenticate: Union[bool | None, Any] = Field(default=None, description="Whether Mailchimp authenticated the Automation")
    """Whether Mailchimp authenticated the Automation"""
    auto_footer: Union[bool | None, Any] = Field(default=None, description="Whether to automatically append Mailchimp's default footer")
    """Whether to automatically append Mailchimp's default footer"""
    inline_css: Union[bool | None, Any] = Field(default=None, description="Whether to automatically inline the CSS")
    """Whether to automatically inline the CSS"""

class AutomationRecipientsSegmentOpts(BaseModel):
    """An object representing all segmentation options"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    saved_segment_id: Union[int | None, Any] = Field(default=None, description="The id for an existing saved segment")
    """The id for an existing saved segment"""
    match: Union[str | None, Any] = Field(default=None, description="Segment match type")
    """Segment match type"""
    conditions: Union[list[dict[str, Any]] | None, Any] = Field(default=None, description="Segment match conditions")
    """Segment match conditions"""

class AutomationRecipients(BaseModel):
    """List settings for the Automation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    list_id: Union[str | None, Any] = Field(default=None, description="The unique list id")
    """The unique list id"""
    list_is_active: Union[bool | None, Any] = Field(default=None, description="The status of the list used")
    """The status of the list used"""
    list_name: Union[str | None, Any] = Field(default=None, description="The name of the list")
    """The name of the list"""
    segment_opts: Union[AutomationRecipientsSegmentOpts | None, Any] = Field(default=None, description="An object representing all segmentation options")
    """An object representing all segmentation options"""
    store_id: Union[str | None, Any] = Field(default=None, description="The id of the store")
    """The id of the store"""

class Automation(BaseModel):
    """A summary of an individual Automation workflow's settings and content"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    create_time: Union[str | None, Any] = Field(default=None)
    start_time: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    emails_sent: Union[int | None, Any] = Field(default=None)
    recipients: Union[AutomationRecipients | None, Any] = Field(default=None)
    settings: Union[AutomationSettings | None, Any] = Field(default=None)
    tracking: Union[AutomationTracking | None, Any] = Field(default=None)
    report_summary: Union[AutomationReportSummary | None, Any] = Field(default=None)

class AutomationsList(BaseModel):
    """A summary of the Automations for an account"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    automations: Union[list[Automation], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class Tag(BaseModel):
    """A tag that can be assigned to a list member"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[int, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)

class TagsList(BaseModel):
    """A list of tags assigned to a list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    tags: Union[list[Tag], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class InterestCategory(BaseModel):
    """Interest categories organize interests, which are used to group subscribers based on their preferences"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    list_id: Union[str | None, Any] = Field(default=None)
    id: Union[str, Any] = Field(default=None)
    title: Union[str | None, Any] = Field(default=None)
    display_order: Union[int | None, Any] = Field(default=None)
    type: Union[str | None, Any] = Field(default=None)

class InterestCategoriesList(BaseModel):
    """Information about this list's interest categories"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    list_id: Union[str | None, Any] = Field(default=None)
    categories: Union[list[InterestCategory], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class Interest(BaseModel):
    """Assign subscribers to interests to group them together"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    category_id: Union[str | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    id: Union[str, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)
    subscriber_count: Union[str | None, Any] = Field(default=None)
    display_order: Union[int | None, Any] = Field(default=None)

class InterestsList(BaseModel):
    """A list of interests for a specific list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    interests: Union[list[Interest], Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    category_id: Union[str | None, Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class SegmentOptions(BaseModel):
    """The conditions of the segment"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    match: Union[str | None, Any] = Field(default=None, description="Match type")
    """Match type"""
    conditions: Union[list[dict[str, Any]] | None, Any] = Field(default=None, description="Segment match conditions")
    """Segment match conditions"""

class Segment(BaseModel):
    """Information about a specific segment"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[int, Any] = Field(default=None)
    name: Union[str | None, Any] = Field(default=None)
    member_count: Union[int | None, Any] = Field(default=None)
    type: Union[str | None, Any] = Field(default=None)
    created_at: Union[str | None, Any] = Field(default=None)
    updated_at: Union[str | None, Any] = Field(default=None)
    options: Union[SegmentOptions | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)

class SegmentsList(BaseModel):
    """A list of available segments"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    segments: Union[list[Segment], Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class SegmentMemberStats(BaseModel):
    """Open and click rates for this subscriber"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    avg_open_rate: Union[float | None, Any] = Field(default=None, description="A subscriber's average open rate")
    """A subscriber's average open rate"""
    avg_click_rate: Union[float | None, Any] = Field(default=None, description="A subscriber's average clickthrough rate")
    """A subscriber's average clickthrough rate"""

class SegmentMemberLocation(BaseModel):
    """Subscriber location information"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    latitude: Union[float | None, Any] = Field(default=None, description="The location latitude")
    """The location latitude"""
    longitude: Union[float | None, Any] = Field(default=None, description="The location longitude")
    """The location longitude"""
    gmtoff: Union[int | None, Any] = Field(default=None, description="The time difference in hours from GMT")
    """The time difference in hours from GMT"""
    dstoff: Union[int | None, Any] = Field(default=None, description="The offset for timezones where daylight saving time is observed")
    """The offset for timezones where daylight saving time is observed"""
    country_code: Union[str | None, Any] = Field(default=None, description="The unique code for the location country")
    """The unique code for the location country"""
    timezone: Union[str | None, Any] = Field(default=None, description="The timezone for the location")
    """The timezone for the location"""
    region: Union[str | None, Any] = Field(default=None, description="The region for the location")
    """The region for the location"""

class SegmentMember(BaseModel):
    """Individuals who are currently or have been previously subscribed to this list"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    id: Union[str, Any] = Field(default=None)
    email_address: Union[str | None, Any] = Field(default=None)
    unique_email_id: Union[str | None, Any] = Field(default=None)
    email_type: Union[str | None, Any] = Field(default=None)
    status: Union[str | None, Any] = Field(default=None)
    merge_fields: Union[dict[str, Any] | None, Any] = Field(default=None)
    interests: Union[dict[str, bool] | None, Any] = Field(default=None)
    stats: Union[SegmentMemberStats | None, Any] = Field(default=None)
    ip_signup: Union[str | None, Any] = Field(default=None)
    timestamp_signup: Union[str | None, Any] = Field(default=None)
    ip_opt: Union[str | None, Any] = Field(default=None)
    timestamp_opt: Union[str | None, Any] = Field(default=None)
    member_rating: Union[int | None, Any] = Field(default=None)
    last_changed: Union[str | None, Any] = Field(default=None)
    language: Union[str | None, Any] = Field(default=None)
    vip: Union[bool | None, Any] = Field(default=None)
    email_client: Union[str | None, Any] = Field(default=None)
    location: Union[SegmentMemberLocation | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)

class SegmentMembersList(BaseModel):
    """View members in a specific list segment"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    members: Union[list[SegmentMember], Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

class Unsubscribe(BaseModel):
    """A member who unsubscribed from a specific campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    email_id: Union[str | None, Any] = Field(default=None)
    email_address: Union[str | None, Any] = Field(default=None)
    merge_fields: Union[dict[str, Any] | None, Any] = Field(default=None)
    vip: Union[bool | None, Any] = Field(default=None)
    timestamp: Union[str | None, Any] = Field(default=None)
    reason: Union[str | None, Any] = Field(default=None)
    campaign_id: Union[str | None, Any] = Field(default=None)
    list_id: Union[str | None, Any] = Field(default=None)
    list_is_active: Union[bool | None, Any] = Field(default=None)

class UnsubscribesList(BaseModel):
    """A list of members who have unsubscribed from a specific campaign"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    unsubscribes: Union[list[Unsubscribe], Any] = Field(default=None)
    campaign_id: Union[str | None, Any] = Field(default=None)
    total_items: Union[int, Any] = Field(default=None)

# ===== METADATA TYPE DEFINITIONS (PYDANTIC) =====
# Meta types for operations that extract metadata (e.g., pagination info)

class CampaignsListResultMeta(BaseModel):
    """Metadata for campaigns.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class ListsListResultMeta(BaseModel):
    """Metadata for lists.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class ListMembersListResultMeta(BaseModel):
    """Metadata for list_members.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class ReportsListResultMeta(BaseModel):
    """Metadata for reports.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class EmailActivityListResultMeta(BaseModel):
    """Metadata for email_activity.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class AutomationsListResultMeta(BaseModel):
    """Metadata for automations.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class TagsListResultMeta(BaseModel):
    """Metadata for tags.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class InterestCategoriesListResultMeta(BaseModel):
    """Metadata for interest_categories.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class InterestsListResultMeta(BaseModel):
    """Metadata for interests.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class SegmentsListResultMeta(BaseModel):
    """Metadata for segments.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class SegmentMembersListResultMeta(BaseModel):
    """Metadata for segment_members.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

class UnsubscribesListResultMeta(BaseModel):
    """Metadata for unsubscribes.Action.LIST operation"""
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    total_items: Union[int, Any] = Field(default=None)

# ===== RESPONSE ENVELOPE MODELS =====

# Type variables for generic envelope models
T = TypeVar('T')
S = TypeVar('S')


class MailchimpExecuteResult(BaseModel, Generic[T]):
    """Response envelope with data only.

    Used for actions that return data without metadata.
    """
    model_config = ConfigDict(extra="forbid")

    data: T
    """Response data containing the result of the action."""


class MailchimpExecuteResultWithMeta(MailchimpExecuteResult[T], Generic[T, S]):
    """Response envelope with data and metadata.

    Used for actions that return both data and metadata (e.g., pagination info).
    """
    meta: S
    """Metadata about the response (e.g., pagination cursors, record counts)."""



# ===== OPERATION RESULT TYPE ALIASES =====

# Concrete type aliases for each operation result.
# These provide simpler, more readable type annotations than using the generic forms.

CampaignsListResult = MailchimpExecuteResultWithMeta[list[Campaign], CampaignsListResultMeta]
"""Result type for campaigns.list operation with data and metadata."""

ListsListResult = MailchimpExecuteResultWithMeta[list[List], ListsListResultMeta]
"""Result type for lists.list operation with data and metadata."""

ListMembersListResult = MailchimpExecuteResultWithMeta[list[ListMember], ListMembersListResultMeta]
"""Result type for list_members.list operation with data and metadata."""

ReportsListResult = MailchimpExecuteResultWithMeta[list[Report], ReportsListResultMeta]
"""Result type for reports.list operation with data and metadata."""

EmailActivityListResult = MailchimpExecuteResultWithMeta[list[EmailActivity], EmailActivityListResultMeta]
"""Result type for email_activity.list operation with data and metadata."""

AutomationsListResult = MailchimpExecuteResultWithMeta[list[Automation], AutomationsListResultMeta]
"""Result type for automations.list operation with data and metadata."""

TagsListResult = MailchimpExecuteResultWithMeta[list[Tag], TagsListResultMeta]
"""Result type for tags.list operation with data and metadata."""

InterestCategoriesListResult = MailchimpExecuteResultWithMeta[list[InterestCategory], InterestCategoriesListResultMeta]
"""Result type for interest_categories.list operation with data and metadata."""

InterestsListResult = MailchimpExecuteResultWithMeta[list[Interest], InterestsListResultMeta]
"""Result type for interests.list operation with data and metadata."""

SegmentsListResult = MailchimpExecuteResultWithMeta[list[Segment], SegmentsListResultMeta]
"""Result type for segments.list operation with data and metadata."""

SegmentMembersListResult = MailchimpExecuteResultWithMeta[list[SegmentMember], SegmentMembersListResultMeta]
"""Result type for segment_members.list operation with data and metadata."""

UnsubscribesListResult = MailchimpExecuteResultWithMeta[list[Unsubscribe], UnsubscribesListResultMeta]
"""Result type for unsubscribes.list operation with data and metadata."""




"""
Mailchimp connector.
"""

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, Callable, TypeVar, overload
try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from .connector_model import MailchimpConnectorModel
from ._vendored.connector_sdk.introspection import describe_entities, generate_tool_description
from .types import (
    AutomationsListParams,
    CampaignsGetParams,
    CampaignsListParams,
    EmailActivityListParams,
    InterestCategoriesGetParams,
    InterestCategoriesListParams,
    InterestsGetParams,
    InterestsListParams,
    ListMembersGetParams,
    ListMembersListParams,
    ListsGetParams,
    ListsListParams,
    ReportsGetParams,
    ReportsListParams,
    SegmentMembersListParams,
    SegmentsGetParams,
    SegmentsListParams,
    TagsListParams,
    UnsubscribesListParams,
)
if TYPE_CHECKING:
    from .models import MailchimpAuthConfig
# Import response models and envelope models at runtime
from .models import (
    MailchimpExecuteResult,
    MailchimpExecuteResultWithMeta,
    CampaignsListResult,
    ListsListResult,
    ListMembersListResult,
    ReportsListResult,
    EmailActivityListResult,
    AutomationsListResult,
    TagsListResult,
    InterestCategoriesListResult,
    InterestsListResult,
    SegmentsListResult,
    SegmentMembersListResult,
    UnsubscribesListResult,
    Automation,
    Campaign,
    EmailActivity,
    Interest,
    InterestCategory,
    List,
    ListMember,
    Report,
    Segment,
    SegmentMember,
    Tag,
    Unsubscribe,
)

# TypeVar for decorator type preservation
_F = TypeVar("_F", bound=Callable[..., Any])



class MailchimpConnector:
    """
    Type-safe Mailchimp API connector.

    Auto-generated from OpenAPI specification with full type safety.
    """

    connector_name = "mailchimp"
    connector_version = "1.0.1"
    vendored_sdk_version = "0.1.0"  # Version of vendored connector-sdk

    # Map of (entity, action) -> needs_envelope for envelope wrapping decision
    _ENVELOPE_MAP = {
        ("campaigns", "list"): True,
        ("campaigns", "get"): None,
        ("lists", "list"): True,
        ("lists", "get"): None,
        ("list_members", "list"): True,
        ("list_members", "get"): None,
        ("reports", "list"): True,
        ("reports", "get"): None,
        ("email_activity", "list"): True,
        ("automations", "list"): True,
        ("tags", "list"): True,
        ("interest_categories", "list"): True,
        ("interest_categories", "get"): None,
        ("interests", "list"): True,
        ("interests", "get"): None,
        ("segments", "list"): True,
        ("segments", "get"): None,
        ("segment_members", "list"): True,
        ("unsubscribes", "list"): True,
    }

    # Map of (entity, action) -> {python_param_name: api_param_name}
    # Used to convert snake_case TypedDict keys to API parameter names in execute()
    _PARAM_MAP = {
        ('campaigns', 'list'): {'count': 'count', 'offset': 'offset', 'type': 'type', 'status': 'status', 'before_send_time': 'before_send_time', 'since_send_time': 'since_send_time', 'before_create_time': 'before_create_time', 'since_create_time': 'since_create_time', 'list_id': 'list_id', 'folder_id': 'folder_id', 'sort_field': 'sort_field', 'sort_dir': 'sort_dir'},
        ('campaigns', 'get'): {'campaign_id': 'campaign_id'},
        ('lists', 'list'): {'count': 'count', 'offset': 'offset', 'before_date_created': 'before_date_created', 'since_date_created': 'since_date_created', 'before_campaign_last_sent': 'before_campaign_last_sent', 'since_campaign_last_sent': 'since_campaign_last_sent', 'email': 'email', 'sort_field': 'sort_field', 'sort_dir': 'sort_dir'},
        ('lists', 'get'): {'list_id': 'list_id'},
        ('list_members', 'list'): {'list_id': 'list_id', 'count': 'count', 'offset': 'offset', 'email_type': 'email_type', 'status': 'status', 'since_timestamp_opt': 'since_timestamp_opt', 'before_timestamp_opt': 'before_timestamp_opt', 'since_last_changed': 'since_last_changed', 'before_last_changed': 'before_last_changed', 'unique_email_id': 'unique_email_id', 'vip_only': 'vip_only', 'interest_category_id': 'interest_category_id', 'interest_ids': 'interest_ids', 'interest_match': 'interest_match', 'sort_field': 'sort_field', 'sort_dir': 'sort_dir'},
        ('list_members', 'get'): {'list_id': 'list_id', 'subscriber_hash': 'subscriber_hash'},
        ('reports', 'list'): {'count': 'count', 'offset': 'offset', 'type': 'type', 'before_send_time': 'before_send_time', 'since_send_time': 'since_send_time'},
        ('reports', 'get'): {'campaign_id': 'campaign_id'},
        ('email_activity', 'list'): {'campaign_id': 'campaign_id', 'count': 'count', 'offset': 'offset', 'since': 'since'},
        ('automations', 'list'): {'count': 'count', 'offset': 'offset', 'before_create_time': 'before_create_time', 'since_create_time': 'since_create_time', 'before_start_time': 'before_start_time', 'since_start_time': 'since_start_time', 'status': 'status'},
        ('tags', 'list'): {'list_id': 'list_id', 'name': 'name'},
        ('interest_categories', 'list'): {'list_id': 'list_id', 'count': 'count', 'offset': 'offset'},
        ('interest_categories', 'get'): {'list_id': 'list_id', 'interest_category_id': 'interest_category_id'},
        ('interests', 'list'): {'list_id': 'list_id', 'interest_category_id': 'interest_category_id', 'count': 'count', 'offset': 'offset'},
        ('interests', 'get'): {'list_id': 'list_id', 'interest_category_id': 'interest_category_id', 'interest_id': 'interest_id'},
        ('segments', 'list'): {'list_id': 'list_id', 'count': 'count', 'offset': 'offset', 'type': 'type', 'since_created_at': 'since_created_at', 'before_created_at': 'before_created_at', 'since_updated_at': 'since_updated_at', 'before_updated_at': 'before_updated_at'},
        ('segments', 'get'): {'list_id': 'list_id', 'segment_id': 'segment_id'},
        ('segment_members', 'list'): {'list_id': 'list_id', 'segment_id': 'segment_id', 'count': 'count', 'offset': 'offset'},
        ('unsubscribes', 'list'): {'campaign_id': 'campaign_id', 'count': 'count', 'offset': 'offset'},
    }

    def __init__(
        self,
        auth_config: MailchimpAuthConfig | None = None,
        external_user_id: str | None = None,
        airbyte_client_id: str | None = None,
        airbyte_client_secret: str | None = None,
        on_token_refresh: Any | None = None,
        data_center: str | None = None    ):
        """
        Initialize a new mailchimp connector instance.

        Supports both local and hosted execution modes:
        - Local mode: Provide `auth_config` for direct API calls
        - Hosted mode: Provide `external_user_id`, `airbyte_client_id`, and `airbyte_client_secret` for hosted execution

        Args:
            auth_config: Typed authentication configuration (required for local mode)
            external_user_id: External user ID (required for hosted mode)
            airbyte_client_id: Airbyte OAuth client ID (required for hosted mode)
            airbyte_client_secret: Airbyte OAuth client secret (required for hosted mode)
            on_token_refresh: Optional callback for OAuth2 token refresh persistence.
                Called with new_tokens dict when tokens are refreshed. Can be sync or async.
                Example: lambda tokens: save_to_database(tokens)            data_center: The data center for your Mailchimp account (e.g., us1, us2, us6)
        Examples:
            # Local mode (direct API calls)
            connector = MailchimpConnector(auth_config=MailchimpAuthConfig(api_key="...", data_center="..."))
            # Hosted mode (executed on Airbyte cloud)
            connector = MailchimpConnector(
                external_user_id="user-123",
                airbyte_client_id="client_abc123",
                airbyte_client_secret="secret_xyz789"
            )

            # Local mode with OAuth2 token refresh callback
            def save_tokens(new_tokens: dict) -> None:
                # Persist updated tokens to your storage (file, database, etc.)
                with open("tokens.json", "w") as f:
                    json.dump(new_tokens, f)

            connector = MailchimpConnector(
                auth_config=MailchimpAuthConfig(access_token="...", refresh_token="..."),
                on_token_refresh=save_tokens
            )
        """
        # Hosted mode: external_user_id, airbyte_client_id, and airbyte_client_secret provided
        if external_user_id and airbyte_client_id and airbyte_client_secret:
            from ._vendored.connector_sdk.executor import HostedExecutor
            self._executor = HostedExecutor(
                external_user_id=external_user_id,
                airbyte_client_id=airbyte_client_id,
                airbyte_client_secret=airbyte_client_secret,
                connector_definition_id=str(MailchimpConnectorModel.id),
            )
        else:
            # Local mode: auth_config required
            if not auth_config:
                raise ValueError(
                    "Either provide (external_user_id, airbyte_client_id, airbyte_client_secret) for hosted mode "
                    "or auth_config for local mode"
                )

            from ._vendored.connector_sdk.executor import LocalExecutor

            # Build config_values dict from server variables
            config_values: dict[str, str] = {}
            if data_center:
                config_values["data_center"] = data_center

            self._executor = LocalExecutor(
                model=MailchimpConnectorModel,
                auth_config=auth_config.model_dump() if auth_config else None,
                config_values=config_values,
                on_token_refresh=on_token_refresh
            )

            # Update base_url with server variables if provided
            base_url = self._executor.http_client.base_url
            if data_center:
                base_url = base_url.replace("{data_center}", data_center)
            self._executor.http_client.base_url = base_url

        # Initialize entity query objects
        self.campaigns = CampaignsQuery(self)
        self.lists = ListsQuery(self)
        self.list_members = ListMembersQuery(self)
        self.reports = ReportsQuery(self)
        self.email_activity = EmailActivityQuery(self)
        self.automations = AutomationsQuery(self)
        self.tags = TagsQuery(self)
        self.interest_categories = InterestCategoriesQuery(self)
        self.interests = InterestsQuery(self)
        self.segments = SegmentsQuery(self)
        self.segment_members = SegmentMembersQuery(self)
        self.unsubscribes = UnsubscribesQuery(self)

    # ===== TYPED EXECUTE METHOD (Recommended Interface) =====

    @overload
    async def execute(
        self,
        entity: Literal["campaigns"],
        action: Literal["list"],
        params: "CampaignsListParams"
    ) -> "CampaignsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["campaigns"],
        action: Literal["get"],
        params: "CampaignsGetParams"
    ) -> "Campaign": ...

    @overload
    async def execute(
        self,
        entity: Literal["lists"],
        action: Literal["list"],
        params: "ListsListParams"
    ) -> "ListsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["lists"],
        action: Literal["get"],
        params: "ListsGetParams"
    ) -> "List": ...

    @overload
    async def execute(
        self,
        entity: Literal["list_members"],
        action: Literal["list"],
        params: "ListMembersListParams"
    ) -> "ListMembersListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["list_members"],
        action: Literal["get"],
        params: "ListMembersGetParams"
    ) -> "ListMember": ...

    @overload
    async def execute(
        self,
        entity: Literal["reports"],
        action: Literal["list"],
        params: "ReportsListParams"
    ) -> "ReportsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["reports"],
        action: Literal["get"],
        params: "ReportsGetParams"
    ) -> "Report": ...

    @overload
    async def execute(
        self,
        entity: Literal["email_activity"],
        action: Literal["list"],
        params: "EmailActivityListParams"
    ) -> "EmailActivityListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["automations"],
        action: Literal["list"],
        params: "AutomationsListParams"
    ) -> "AutomationsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["tags"],
        action: Literal["list"],
        params: "TagsListParams"
    ) -> "TagsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["interest_categories"],
        action: Literal["list"],
        params: "InterestCategoriesListParams"
    ) -> "InterestCategoriesListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["interest_categories"],
        action: Literal["get"],
        params: "InterestCategoriesGetParams"
    ) -> "InterestCategory": ...

    @overload
    async def execute(
        self,
        entity: Literal["interests"],
        action: Literal["list"],
        params: "InterestsListParams"
    ) -> "InterestsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["interests"],
        action: Literal["get"],
        params: "InterestsGetParams"
    ) -> "Interest": ...

    @overload
    async def execute(
        self,
        entity: Literal["segments"],
        action: Literal["list"],
        params: "SegmentsListParams"
    ) -> "SegmentsListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["segments"],
        action: Literal["get"],
        params: "SegmentsGetParams"
    ) -> "Segment": ...

    @overload
    async def execute(
        self,
        entity: Literal["segment_members"],
        action: Literal["list"],
        params: "SegmentMembersListParams"
    ) -> "SegmentMembersListResult": ...

    @overload
    async def execute(
        self,
        entity: Literal["unsubscribes"],
        action: Literal["list"],
        params: "UnsubscribesListParams"
    ) -> "UnsubscribesListResult": ...


    @overload
    async def execute(
        self,
        entity: str,
        action: str,
        params: dict[str, Any]
    ) -> MailchimpExecuteResult[Any] | MailchimpExecuteResultWithMeta[Any, Any] | Any: ...

    async def execute(
        self,
        entity: str,
        action: str,
        params: dict[str, Any] | None = None
    ) -> Any:
        """
        Execute an entity operation with full type safety.

        This is the recommended interface for blessed connectors as it:
        - Uses the same signature as non-blessed connectors
        - Provides full IDE autocomplete for entity/action/params
        - Makes migration from generic to blessed connectors seamless

        Args:
            entity: Entity name (e.g., "customers")
            action: Operation action (e.g., "create", "get", "list")
            params: Operation parameters (typed based on entity+action)

        Returns:
            Typed response based on the operation

        Example:
            customer = await connector.execute(
                entity="customers",
                action="get",
                params={"id": "cus_123"}
            )
        """
        from ._vendored.connector_sdk.executor import ExecutionConfig

        # Remap parameter names from snake_case (TypedDict keys) to API parameter names
        if params:
            param_map = self._PARAM_MAP.get((entity, action), {})
            if param_map:
                params = {param_map.get(k, k): v for k, v in params.items()}

        # Use ExecutionConfig for both local and hosted executors
        config = ExecutionConfig(
            entity=entity,
            action=action,
            params=params
        )

        result = await self._executor.execute(config)

        if not result.success:
            raise RuntimeError(f"Execution failed: {result.error}")

        # Check if this operation has extractors configured
        has_extractors = self._ENVELOPE_MAP.get((entity, action), False)

        if has_extractors:
            # With extractors - return Pydantic envelope with data and meta
            if result.meta is not None:
                return MailchimpExecuteResultWithMeta[Any, Any](
                    data=result.data,
                    meta=result.meta
                )
            else:
                return MailchimpExecuteResult[Any](data=result.data)
        else:
            # No extractors - return raw response data
            return result.data

    # ===== INTROSPECTION METHODS =====

    @classmethod
    def describe(cls, func: _F) -> _F:
        """
        Decorator that populates a function's docstring with connector capabilities.

        This class method can be used as a decorator to automatically generate
        comprehensive documentation for AI tool functions.

        Usage:
            @mcp.tool()
            @MailchimpConnector.describe
            async def execute(entity: str, action: str, params: dict):
                '''Execute operations.'''
                ...

        The decorated function's __doc__ will be updated with:
        - Available entities and their actions
        - Parameter signatures with required (*) and optional (?) markers
        - Response structure documentation
        - Example questions (if available in OpenAPI spec)

        Args:
            func: The function to decorate

        Returns:
            The same function with updated __doc__
        """
        description = generate_tool_description(MailchimpConnectorModel)

        original_doc = func.__doc__ or ""
        if original_doc.strip():
            func.__doc__ = f"{original_doc.strip()}\n{description}"
        else:
            func.__doc__ = description

        return func

    def list_entities(self) -> list[dict[str, Any]]:
        """
        Get structured data about available entities, actions, and parameters.

        Returns a list of entity descriptions with:
        - entity_name: Name of the entity (e.g., "contacts", "deals")
        - description: Entity description from the first endpoint
        - available_actions: List of actions (e.g., ["list", "get", "create"])
        - parameters: Dict mapping action -> list of parameter dicts

        Example:
            entities = connector.list_entities()
            for entity in entities:
                print(f"{entity['entity_name']}: {entity['available_actions']}")
        """
        return describe_entities(MailchimpConnectorModel)

    def entity_schema(self, entity: str) -> dict[str, Any] | None:
        """
        Get the JSON schema for an entity.

        Args:
            entity: Entity name (e.g., "contacts", "companies")

        Returns:
            JSON schema dict describing the entity structure, or None if not found.

        Example:
            schema = connector.entity_schema("contacts")
            if schema:
                print(f"Contact properties: {list(schema.get('properties', {}).keys())}")
        """
        entity_def = next(
            (e for e in MailchimpConnectorModel.entities if e.name == entity),
            None
        )
        if entity_def is None:
            logging.getLogger(__name__).warning(
                f"Entity '{entity}' not found. Available entities: "
                f"{[e.name for e in MailchimpConnectorModel.entities]}"
            )
        return entity_def.entity_schema if entity_def else None



class CampaignsQuery:
    """
    Query class for Campaigns entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        count: int | None = None,
        offset: int | None = None,
        type: str | None = None,
        status: str | None = None,
        before_send_time: str | None = None,
        since_send_time: str | None = None,
        before_create_time: str | None = None,
        since_create_time: str | None = None,
        list_id: str | None = None,
        folder_id: str | None = None,
        sort_field: str | None = None,
        sort_dir: str | None = None,
        **kwargs
    ) -> CampaignsListResult:
        """
        Get all campaigns in an account

        Args:
            count: The number of records to return. Default is 10. Maximum is 1000.
            offset: Used for pagination, this is the number of records from a collection to skip.
            type: The campaign type
            status: The status of the campaign
            before_send_time: Restrict the response to campaigns sent before the set time
            since_send_time: Restrict the response to campaigns sent after the set time
            before_create_time: Restrict the response to campaigns created before the set time
            since_create_time: Restrict the response to campaigns created after the set time
            list_id: The unique id for the list
            folder_id: The unique folder id
            sort_field: Returns files sorted by the specified field
            sort_dir: Determines the order direction for sorted results
            **kwargs: Additional parameters

        Returns:
            CampaignsListResult
        """
        params = {k: v for k, v in {
            "count": count,
            "offset": offset,
            "type": type,
            "status": status,
            "before_send_time": before_send_time,
            "since_send_time": since_send_time,
            "before_create_time": before_create_time,
            "since_create_time": since_create_time,
            "list_id": list_id,
            "folder_id": folder_id,
            "sort_field": sort_field,
            "sort_dir": sort_dir,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("campaigns", "list", params)
        # Cast generic envelope to concrete typed result
        return CampaignsListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        campaign_id: str,
        **kwargs
    ) -> Campaign:
        """
        Get information about a specific campaign

        Args:
            campaign_id: The unique id for the campaign
            **kwargs: Additional parameters

        Returns:
            Campaign
        """
        params = {k: v for k, v in {
            "campaign_id": campaign_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("campaigns", "get", params)
        return result



class ListsQuery:
    """
    Query class for Lists entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        count: int | None = None,
        offset: int | None = None,
        before_date_created: str | None = None,
        since_date_created: str | None = None,
        before_campaign_last_sent: str | None = None,
        since_campaign_last_sent: str | None = None,
        email: str | None = None,
        sort_field: str | None = None,
        sort_dir: str | None = None,
        **kwargs
    ) -> ListsListResult:
        """
        Get information about all lists in the account

        Args:
            count: The number of records to return
            offset: Used for pagination
            before_date_created: Restrict response to lists created before the set date
            since_date_created: Restrict response to lists created after the set date
            before_campaign_last_sent: Restrict results to lists created before the last campaign send date
            since_campaign_last_sent: Restrict results to lists created after the last campaign send date
            email: Restrict results to lists that include a specific subscriber's email address
            sort_field: Returns files sorted by the specified field
            sort_dir: Determines the order direction for sorted results
            **kwargs: Additional parameters

        Returns:
            ListsListResult
        """
        params = {k: v for k, v in {
            "count": count,
            "offset": offset,
            "before_date_created": before_date_created,
            "since_date_created": since_date_created,
            "before_campaign_last_sent": before_campaign_last_sent,
            "since_campaign_last_sent": since_campaign_last_sent,
            "email": email,
            "sort_field": sort_field,
            "sort_dir": sort_dir,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("lists", "list", params)
        # Cast generic envelope to concrete typed result
        return ListsListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        list_id: str,
        **kwargs
    ) -> List:
        """
        Get information about a specific list in your Mailchimp account

        Args:
            list_id: The unique ID for the list
            **kwargs: Additional parameters

        Returns:
            List
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("lists", "get", params)
        return result



class ListMembersQuery:
    """
    Query class for ListMembers entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        count: int | None = None,
        offset: int | None = None,
        email_type: str | None = None,
        status: str | None = None,
        since_timestamp_opt: str | None = None,
        before_timestamp_opt: str | None = None,
        since_last_changed: str | None = None,
        before_last_changed: str | None = None,
        unique_email_id: str | None = None,
        vip_only: bool | None = None,
        interest_category_id: str | None = None,
        interest_ids: str | None = None,
        interest_match: str | None = None,
        sort_field: str | None = None,
        sort_dir: str | None = None,
        **kwargs
    ) -> ListMembersListResult:
        """
        Get information about members in a specific Mailchimp list

        Args:
            list_id: The unique ID for the list
            count: The number of records to return
            offset: Used for pagination
            email_type: The email type
            status: The subscriber's status
            since_timestamp_opt: Restrict results to subscribers who opted-in after the set timeframe
            before_timestamp_opt: Restrict results to subscribers who opted-in before the set timeframe
            since_last_changed: Restrict results to subscribers whose information changed after the set timeframe
            before_last_changed: Restrict results to subscribers whose information changed before the set timeframe
            unique_email_id: A unique identifier for the email address across all Mailchimp lists
            vip_only: A filter to return only the list's VIP members
            interest_category_id: The unique id for the interest category
            interest_ids: Used to filter list members by interests
            interest_match: Used to filter list members by interests
            sort_field: Returns files sorted by the specified field
            sort_dir: Determines the order direction for sorted results
            **kwargs: Additional parameters

        Returns:
            ListMembersListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "count": count,
            "offset": offset,
            "email_type": email_type,
            "status": status,
            "since_timestamp_opt": since_timestamp_opt,
            "before_timestamp_opt": before_timestamp_opt,
            "since_last_changed": since_last_changed,
            "before_last_changed": before_last_changed,
            "unique_email_id": unique_email_id,
            "vip_only": vip_only,
            "interest_category_id": interest_category_id,
            "interest_ids": interest_ids,
            "interest_match": interest_match,
            "sort_field": sort_field,
            "sort_dir": sort_dir,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("list_members", "list", params)
        # Cast generic envelope to concrete typed result
        return ListMembersListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        list_id: str,
        subscriber_hash: str,
        **kwargs
    ) -> ListMember:
        """
        Get information about a specific list member

        Args:
            list_id: The unique ID for the list
            subscriber_hash: The MD5 hash of the lowercase version of the list member's email address
            **kwargs: Additional parameters

        Returns:
            ListMember
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "subscriber_hash": subscriber_hash,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("list_members", "get", params)
        return result



class ReportsQuery:
    """
    Query class for Reports entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        count: int | None = None,
        offset: int | None = None,
        type: str | None = None,
        before_send_time: str | None = None,
        since_send_time: str | None = None,
        **kwargs
    ) -> ReportsListResult:
        """
        Get campaign reports

        Args:
            count: The number of records to return
            offset: Used for pagination
            type: The campaign type
            before_send_time: Restrict the response to campaigns sent before the set time
            since_send_time: Restrict the response to campaigns sent after the set time
            **kwargs: Additional parameters

        Returns:
            ReportsListResult
        """
        params = {k: v for k, v in {
            "count": count,
            "offset": offset,
            "type": type,
            "before_send_time": before_send_time,
            "since_send_time": since_send_time,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("reports", "list", params)
        # Cast generic envelope to concrete typed result
        return ReportsListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        campaign_id: str,
        **kwargs
    ) -> Report:
        """
        Get report details for a specific sent campaign

        Args:
            campaign_id: The unique id for the campaign
            **kwargs: Additional parameters

        Returns:
            Report
        """
        params = {k: v for k, v in {
            "campaign_id": campaign_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("reports", "get", params)
        return result



class EmailActivityQuery:
    """
    Query class for EmailActivity entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        campaign_id: str,
        count: int | None = None,
        offset: int | None = None,
        since: str | None = None,
        **kwargs
    ) -> EmailActivityListResult:
        """
        Get a list of member's subscriber activity in a specific campaign

        Args:
            campaign_id: The unique id for the campaign
            count: The number of records to return
            offset: Used for pagination
            since: Restrict results to email activity events that occur after a specific time
            **kwargs: Additional parameters

        Returns:
            EmailActivityListResult
        """
        params = {k: v for k, v in {
            "campaign_id": campaign_id,
            "count": count,
            "offset": offset,
            "since": since,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("email_activity", "list", params)
        # Cast generic envelope to concrete typed result
        return EmailActivityListResult(
            data=result.data,
            meta=result.meta
        )



class AutomationsQuery:
    """
    Query class for Automations entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        count: int | None = None,
        offset: int | None = None,
        before_create_time: str | None = None,
        since_create_time: str | None = None,
        before_start_time: str | None = None,
        since_start_time: str | None = None,
        status: str | None = None,
        **kwargs
    ) -> AutomationsListResult:
        """
        Get a summary of an account's classic automations

        Args:
            count: The number of records to return
            offset: Used for pagination
            before_create_time: Restrict the response to automations created before this time
            since_create_time: Restrict the response to automations created after this time
            before_start_time: Restrict the response to automations started before this time
            since_start_time: Restrict the response to automations started after this time
            status: Restrict the results to automations with the specified status
            **kwargs: Additional parameters

        Returns:
            AutomationsListResult
        """
        params = {k: v for k, v in {
            "count": count,
            "offset": offset,
            "before_create_time": before_create_time,
            "since_create_time": since_create_time,
            "before_start_time": before_start_time,
            "since_start_time": since_start_time,
            "status": status,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("automations", "list", params)
        # Cast generic envelope to concrete typed result
        return AutomationsListResult(
            data=result.data,
            meta=result.meta
        )



class TagsQuery:
    """
    Query class for Tags entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        name: str | None = None,
        **kwargs
    ) -> TagsListResult:
        """
        Search for tags on a list by name

        Args:
            list_id: The unique ID for the list
            name: The search query used to filter tags
            **kwargs: Additional parameters

        Returns:
            TagsListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "name": name,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("tags", "list", params)
        # Cast generic envelope to concrete typed result
        return TagsListResult(
            data=result.data,
            meta=result.meta
        )



class InterestCategoriesQuery:
    """
    Query class for InterestCategories entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        count: int | None = None,
        offset: int | None = None,
        **kwargs
    ) -> InterestCategoriesListResult:
        """
        Get information about a list's interest categories

        Args:
            list_id: The unique ID for the list
            count: The number of records to return
            offset: Used for pagination
            **kwargs: Additional parameters

        Returns:
            InterestCategoriesListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "count": count,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("interest_categories", "list", params)
        # Cast generic envelope to concrete typed result
        return InterestCategoriesListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        list_id: str,
        interest_category_id: str,
        **kwargs
    ) -> InterestCategory:
        """
        Get information about a specific interest category

        Args:
            list_id: The unique ID for the list
            interest_category_id: The unique ID for the interest category
            **kwargs: Additional parameters

        Returns:
            InterestCategory
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "interest_category_id": interest_category_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("interest_categories", "get", params)
        return result



class InterestsQuery:
    """
    Query class for Interests entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        interest_category_id: str,
        count: int | None = None,
        offset: int | None = None,
        **kwargs
    ) -> InterestsListResult:
        """
        Get a list of this category's interests

        Args:
            list_id: The unique ID for the list
            interest_category_id: The unique ID for the interest category
            count: The number of records to return
            offset: Used for pagination
            **kwargs: Additional parameters

        Returns:
            InterestsListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "interest_category_id": interest_category_id,
            "count": count,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("interests", "list", params)
        # Cast generic envelope to concrete typed result
        return InterestsListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        list_id: str,
        interest_category_id: str,
        interest_id: str,
        **kwargs
    ) -> Interest:
        """
        Get interests or group names for a specific category

        Args:
            list_id: The unique ID for the list
            interest_category_id: The unique ID for the interest category
            interest_id: The specific interest or group name
            **kwargs: Additional parameters

        Returns:
            Interest
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "interest_category_id": interest_category_id,
            "interest_id": interest_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("interests", "get", params)
        return result



class SegmentsQuery:
    """
    Query class for Segments entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        count: int | None = None,
        offset: int | None = None,
        type: str | None = None,
        since_created_at: str | None = None,
        before_created_at: str | None = None,
        since_updated_at: str | None = None,
        before_updated_at: str | None = None,
        **kwargs
    ) -> SegmentsListResult:
        """
        Get information about all available segments for a specific list

        Args:
            list_id: The unique ID for the list
            count: The number of records to return
            offset: Used for pagination
            type: Limit results based on segment type
            since_created_at: Restrict results to segments created after the set time
            before_created_at: Restrict results to segments created before the set time
            since_updated_at: Restrict results to segments updated after the set time
            before_updated_at: Restrict results to segments updated before the set time
            **kwargs: Additional parameters

        Returns:
            SegmentsListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "count": count,
            "offset": offset,
            "type": type,
            "since_created_at": since_created_at,
            "before_created_at": before_created_at,
            "since_updated_at": since_updated_at,
            "before_updated_at": before_updated_at,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("segments", "list", params)
        # Cast generic envelope to concrete typed result
        return SegmentsListResult(
            data=result.data,
            meta=result.meta
        )



    async def get(
        self,
        list_id: str,
        segment_id: str,
        **kwargs
    ) -> Segment:
        """
        Get information about a specific segment

        Args:
            list_id: The unique ID for the list
            segment_id: The unique id for the segment
            **kwargs: Additional parameters

        Returns:
            Segment
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "segment_id": segment_id,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("segments", "get", params)
        return result



class SegmentMembersQuery:
    """
    Query class for SegmentMembers entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        list_id: str,
        segment_id: str,
        count: int | None = None,
        offset: int | None = None,
        **kwargs
    ) -> SegmentMembersListResult:
        """
        Get information about members in a saved segment

        Args:
            list_id: The unique ID for the list
            segment_id: The unique id for the segment
            count: The number of records to return
            offset: Used for pagination
            **kwargs: Additional parameters

        Returns:
            SegmentMembersListResult
        """
        params = {k: v for k, v in {
            "list_id": list_id,
            "segment_id": segment_id,
            "count": count,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("segment_members", "list", params)
        # Cast generic envelope to concrete typed result
        return SegmentMembersListResult(
            data=result.data,
            meta=result.meta
        )



class UnsubscribesQuery:
    """
    Query class for Unsubscribes entity operations.
    """

    def __init__(self, connector: MailchimpConnector):
        """Initialize query with connector reference."""
        self._connector = connector

    async def list(
        self,
        campaign_id: str,
        count: int | None = None,
        offset: int | None = None,
        **kwargs
    ) -> UnsubscribesListResult:
        """
        Get information about members who have unsubscribed from a specific campaign

        Args:
            campaign_id: The unique id for the campaign
            count: The number of records to return
            offset: Used for pagination
            **kwargs: Additional parameters

        Returns:
            UnsubscribesListResult
        """
        params = {k: v for k, v in {
            "campaign_id": campaign_id,
            "count": count,
            "offset": offset,
            **kwargs
        }.items() if v is not None}

        result = await self._connector.execute("unsubscribes", "list", params)
        # Cast generic envelope to concrete typed result
        return UnsubscribesListResult(
            data=result.data,
            meta=result.meta
        )


